@@ -1,3 +1,4 @@
+'''Running subprocesses asynchronously.'''
 import fcntl
 import os
 import os.path
@@ -15,6 +16,7 @@ from seesaw.config import realize
 
 
 class AsyncPopen(object):
+    '''Asynchronous version of :class:`subprocess.Popen`.'''
     def __init__(self, *args, **kwargs):
         self.args = args
         self.kwargs = kwargs
@@ -32,12 +34,8 @@ class AsyncPopen(object):
         self.master_fd = master_fd
         self.master = os.fdopen(master_fd)
 
-        # check for process exit
-        self.wait_callback = PeriodicCallback(self._wait_for_end, 250)
-
         # listen to stdout, stderr
         self.ioloop.add_handler(master_fd, self._handle_subprocess_stdout, self.ioloop.READ)
-        self.wait_callback.start()
 
         slave = os.fdopen(slave_fd)
         self.kwargs["stdout"] = slave
@@ -47,6 +45,10 @@ class AsyncPopen(object):
 
         self.stdin = self.pipe.stdin
 
+        # check for process exit
+        self.wait_callback = PeriodicCallback(self._wait_for_end, 250)
+        self.wait_callback.start()
+
     def _handle_subprocess_stdout(self, fd, events):
         if not self.master.closed and (events & IOLoop._EPOLLIN) != 0:
             data = self.master.read()
@@ -64,6 +66,7 @@ class AsyncPopen(object):
 
 
 class ExternalProcess(Task):
+    '''External subprocess runner.'''
     def __init__(self, name, args, max_tries=1, retry_delay=30, accept_on_exit_code=[0], retry_on_exit_code=None, env=None):
         Task.__init__(self, name)
         self.args = args
@@ -71,12 +74,16 @@ class ExternalProcess(Task):
         self.retry_delay = retry_delay
         self.accept_on_exit_code = accept_on_exit_code
         self.retry_on_exit_code = retry_on_exit_code
-        self.env = env
+        self.env = env or {}
+
+        if 'PYTHONIOENCODING' not in self.env:
+            self.env['PYTHONIOENCODING'] = 'utf8:replace'
 
     def enqueue(self, item):
         self.start_item(item)
         item.log_output("Starting %s for %s\n" % (self, item.description()))
-        item["tries"] = 1
+        item["tries"] = 0
+        item["ExternalProcess.stdin_write_error"] = False
         self.process(item)
 
     def stdin_data(self, item):
@@ -96,14 +103,21 @@ class ExternalProcess(Task):
 
             p.run()
 
-            p.stdin.write(self.stdin_data(item))
+            try:
+                p.stdin.write(self.stdin_data(item))
+            except Exception as error:
+                # FIXME: We need to properly propagate errors
+                item.log_output("Error writing to process: %s" % str(error))
+                item["ExternalProcess.stdin_write_error"] = True
+
             p.stdin.close()
 
     def on_subprocess_stdout(self, pipe, item, data):
         item.log_output(data, full_line=False)
 
     def on_subprocess_end(self, item, returncode):
-        if returncode in self.accept_on_exit_code:
+        if returncode in self.accept_on_exit_code and \
+        not item["ExternalProcess.stdin_write_error"]:
             self.handle_process_result(returncode, item)
         else:
             self.handle_process_error(returncode, item)
@@ -115,13 +129,25 @@ class ExternalProcess(Task):
     def handle_process_error(self, exit_code, item):
         item["tries"] += 1
 
-        item.log_output("Process %s returned exit code %d for %s\n" % (self, exit_code, item.description()))
+        item.log_output("Process %s returned exit code %d for %s\n" %
+            (self, exit_code, item.description())
+        )
         item.log_error(self, exit_code)
 
-        if (self.max_tries == None or item["tries"] < self.max_tries) and (self.retry_on_exit_code == None or exit_code in self.retry_on_exit_code):
-            item.log_output("Retrying %s for %s after %d seconds...\n" % (self, item.description(), self.retry_delay))
-            IOLoop.instance().add_timeout(datetime.timedelta(seconds=self.retry_delay),
-                functools.partial(self.process, item))
+        retry_acceptable = self.max_tries == None or \
+            item["tries"] < self.max_tries
+        exit_status_indicates_retry = self.retry_on_exit_code == None or \
+            exit_code in self.retry_on_exit_code or \
+            item["ExternalProcess.stdin_write_error"]
+
+        if retry_acceptable and exit_status_indicates_retry:
+            item.log_output("Retrying %s for %s after %d seconds...\n" %
+                (self, item.description(), self.retry_delay)
+            )
+            IOLoop.instance().add_timeout(
+                datetime.timedelta(seconds=self.retry_delay),
+                functools.partial(self.process, item)
+            )
 
         else:
             item.log_output("Failed %s for %s\n" % (self, item.description()))
@@ -129,6 +155,7 @@ class ExternalProcess(Task):
 
 
 class WgetDownload(ExternalProcess):
+    '''Download with Wget process runner.'''
     def __init__(self, args, max_tries=1, accept_on_exit_code=[0], retry_on_exit_code=None, env=None, stdin_data_function=None):
         ExternalProcess.__init__(self, "WgetDownload",
             args=args, max_tries=max_tries,
@@ -145,7 +172,9 @@ class WgetDownload(ExternalProcess):
 
 
 class RsyncUpload(ExternalProcess):
-    def __init__(self, target, files, target_source_path="./", bwlimit="0", max_tries=None, extra_args=[]):
+    '''Upload with Rsync process runner.'''
+    def __init__(self, target, files, target_source_path="./", bwlimit="0",
+    max_tries=None, extra_args=[]):
         args = [
           "rsync",
           "-avz",
@@ -169,10 +198,17 @@ class RsyncUpload(ExternalProcess):
         self.target_source_path = target_source_path
 
     def stdin_data(self, item):
-        return "".join(["%s\n" % os.path.relpath(realize(f, item), realize(self.target_source_path, item)) for f in self.files])
+        return "".join([
+            "%s\n" % os.path.relpath(
+                realize(f, item),
+                realize(self.target_source_path, item)
+            )
+            for f in realize(self.files)
+        ])
 
 
 class CurlUpload(ExternalProcess):
+    '''Upload with Curl process runner.'''
     def __init__(self, target, filename, connect_timeout="60", speed_limit="1", speed_time="900", max_tries=None):
         args = [
           "curl",
