@@ -212,6 +212,10 @@ $(function() {
     $('#settings-list').load('/api/settings');
   }
 
+  function reloadHelpTab() {
+    $('#help').load('/api/help')
+  }
+
   var warriorStatus = {
     'NO_PROJECT': ['The warrior is idle. Select a project.', 'Shut down', '/api/stop'],
     'INVALID_SETTINGS': ['You must configure the warrior.', 'Shut down', '/api/stop'],
@@ -353,7 +357,11 @@ $(function() {
 
     h3 = document.createElement('h3');
     $(h3).append($("<span>", { "class": 'twisty' }),
-                 $("<span>", { "class": 'name', text: item.name }),
+                 $("<span>", {
+                     "class": 'name',
+                     text: item.name,
+                     title: item.project,
+                 }),
                  $("<span>", { "class": 'status-line' }),
                  $("<span>", { "class": 'log-line' }));
     itemDiv.appendChild(h3);
@@ -538,6 +546,8 @@ $(function() {
       reloadProjectsTab();
     else if (view=='view-settings')
       reloadSettingsTab();
+    else if (view=='view-help')
+      reloadHelpTab();
   }
 
   $('#tabs').click(function(e) {
